using System;
using System.Drawing;
using System.Windows.Forms;

class MBController : Panel
{
	// Members
	private Mandelbrot mb;
	private MBView mv;
	private Label lDepth;
	private TrackBar depth;
	private TextBox xMin;
	private TextBox xMax;
	private TextBox yMin;
	private TextBox yMax;
	public Button mbControl;
	private ComboBox mbStartPattern;
	private bool isRunning;
	
	// C'tor
	public MBController(Mandelbrot mb, MBView mv)
	{
		this.mb = mb;
		this.mv = mv;
		this.isRunning = false;
		
		// Add controls
		int margin = 5;
		int labelWidth = 50;
		int tbWidth = 60;
		HorizontalAlignment haR = HorizontalAlignment.Right;
		ContentAlignment caR = ContentAlignment.TopRight;

		this.mbControl = new Button();
		this.mbControl.Location = new Point(margin, margin);
		this.mbControl.Width = tbWidth + 30;
		this.mbControl.Text = "Start";
		this.mbControl.Click += new EventHandler(this.controlMb);
		this.Controls.Add(this.mbControl);
		
		this.lDepth = new Label();
		this.lDepth.Location = new Point(this.mbControl.Right + margin, margin);
		this.lDepth.Width = labelWidth + 20;
		this.lDepth.Text = "Depth: 100";
		this.Controls.Add(this.lDepth);
		
		this.depth = new TrackBar();
		this.depth.Location = new Point(this.lDepth.Right + margin, margin);
		this.depth.Width = (4 * labelWidth + 5 * tbWidth + 9 * margin + 30) - (tbWidth + 30 + labelWidth + 4 * margin);
		this.depth.Minimum = 0;
		this.depth.Maximum = 1000;
		this.depth.Value = 100;
		this.depth.ValueChanged += new EventHandler(this.changeDepth);
		this.Controls.Add(this.depth);

		this.mbStartPattern = new ComboBox();
		this.mbStartPattern.Location = new Point(margin, this.depth.Bottom + margin);
		this.mbStartPattern.Width = tbWidth + 30;
		
		for (int i=0; i<3; i++)
			this.mbStartPattern.Items.Add("Pattern " + i.ToString());
		
		this.mbStartPattern.SelectedIndex = 0;
		this.mbStartPattern.DropDownStyle = ComboBoxStyle.DropDownList;
		this.mbStartPattern.SelectedIndexChanged += new EventHandler(this.selectPattern);
		
		this.Controls.Add(this.mbStartPattern);
		
		Label l = new Label();
		l.Location = new Point(this.mbStartPattern.Right + margin, this.mbStartPattern.Top);
		l.Width = labelWidth;
		l.TextAlign = caR;
		l.Text = "X Min:";
		this.Controls.Add(l);
		
		this.xMin = new TextBox();
		this.xMin.Location = new Point(l.Right + margin, l.Top);
		this.xMin.Width = tbWidth;
		this.xMin.TextAlign = haR;
		this.xMin.Text = "-2,25";
		this.Controls.Add(this.xMin);

		l = new Label();
		l.Location = new Point(this.xMin.Right + margin, this.xMin.Top);
		l.Width = labelWidth;
		l.TextAlign = caR;
		l.Text = "X Max:";
		this.Controls.Add(l);

		this.xMax = new TextBox();
		this.xMax.Location = new Point(l.Right + margin, l.Top);
		this.xMax.Width = tbWidth;
		this.xMax.TextAlign = haR;
		this.xMax.Text = "1,00";
		this.Controls.Add(this.xMax);

		l = new Label();
		l.Location = new Point(this.xMax.Right + margin, this.xMax.Top);
		l.Width = labelWidth;
		l.TextAlign = caR;
		l.Text = "Y Min: i";
		this.Controls.Add(l);

		this.yMin = new TextBox();
		this.yMin.Location = new Point(l.Right + margin, l.Top);
		this.yMin.Width = tbWidth;
		this.yMin.TextAlign = haR;
		this.yMin.Text = "-1,25";
		this.Controls.Add(this.yMin);

		l = new Label();
		l.Location = new Point(this.yMin.Right + margin, this.yMin.Top);
		l.Width = labelWidth;
		l.TextAlign = caR;
		l.Text = "Y Max: i";
		this.Controls.Add(l);

		this.yMax = new TextBox();
		this.yMax.Location = new Point(l.Right + margin, l.Top);
		this.yMax.Width = tbWidth;
		this.yMax.TextAlign = haR;
		this.yMax.Text = "1,5";
		this.Controls.Add(this.yMax);

		this.mv.setSizeValues(-2.25, 1.0, -1.25, 1.5);

		this.Height = l.Bottom + margin;
	}
	
	// Methods
	private void controlMb(object o, EventArgs e)
	{
		if (this.isRunning)
		{
			this.mbControl.Text = "Start";
			this.Cursor = Cursors.Default;
			this.mv.StopWaiting();
			this.mb.Stop();
		}
		else
		{
			this.mbControl.Text = "Cancel";
			this.Cursor = Cursors.WaitCursor;
			this.mv.Wait();
			
			double XMin = 0;
			double XMax = 0;
			double YMin = 0;
			double YMax = 0;
			
			// catch the x / y values either from the mouseRect or the Textboxes
			if (this.mv.XMin != 0)
			{
				XMin = this.mv.XMin;
				XMax = this.mv.XMax;
				YMin = this.mv.YMin;
				YMax = this.mv.YMax;
				
				this.xMin.Text = XMin.ToString();
				this.xMax.Text = XMax.ToString();
				this.yMin.Text = YMin.ToString();
				this.yMax.Text = YMax.ToString();
			}
			else
			{
				try
				{
					XMin = Double.Parse(this.xMin.Text);
					XMax = Double.Parse(this.xMax.Text);
					YMin = Double.Parse(this.yMin.Text);
					YMax = Double.Parse(this.yMax.Text);
				}
				catch (ArgumentNullException)
				{
					MessageBox.Show("Die eingetragenen Werte sind ungltig.", "Mandelbrot", MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
				catch (FormatException)
				{
					MessageBox.Show("Die eingetragenen Werte sind ungltig.", "Mandelbrot", MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
				catch (OverflowException)
				{
					MessageBox.Show("Die eingetragenen Werte sind ungltig.", "Mandelbrot", MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
				catch (ArgumentException)
				{
					MessageBox.Show("Die eingetragenen Werte sind ungltig.", "Mandelbrot", MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
			}
			
			this.mb.Start(Mandelbrot.CreateInfo(
					this.depth.Value, 
					XMin, 
					XMax, 
					YMin, 
					YMax, 
					this.mv.ClientRectangle.Size));
					
			this.mv.setSizeValues(XMin, XMax, YMin, YMax);
		}
		
		this.isRunning = !this.isRunning;
	}
	
	public void MandelbrotStopped()
	{
		EventHandler e = new EventHandler(this.controlMb);
		this.Invoke(e, new object[] {null, null});
	}
	
	private void changeDepth(object o, EventArgs e)
	{
		this.lDepth.Text = "Depth: " + this.depth.Value.ToString();
	}
	
	private void selectPattern(object o, EventArgs e)
	{
		switch (this.mbStartPattern.SelectedIndex)
		{
			case 1:
				this.xMin.Text = "-0,745468";
				this.xMax.Text = "-0,745385";
				this.yMin.Text = "0,112975";
				this.yMax.Text = "0,113044";
				this.depth.Value = 200;
				break;
			case 2:
				this.xMin.Text = "-0,74581";
				this.xMax.Text = "-0,74448";
				this.yMin.Text = "0,11196";
				this.yMax.Text = "0,11339";
				this.depth.Value = 200;
				break;
			case 0:
			default:
				this.xMin.Text = "-2,25";
				this.xMax.Text = "1,0";
				this.yMin.Text = "-1,25";
				this.yMax.Text = "1,5";
				this.depth.Value = 100;
				break;			
		}
	}
}